"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Persist = exports.TYPE_REHYDRATE = void 0;
var tslib_1 = require("tslib");
var memoryStorage_1 = tslib_1.__importDefault(require("../storages/memoryStorage"));
exports.TYPE_REHYDRATE = 'ReduxModel/rehydrate';
var Persist = /** @class */ (function () {
    function Persist(storeHelper) {
        this.keyPrefix = 'ReduxModel:Persist:';
        this.bootstrapped = false;
        this.allowKeys = [];
        this.mapFromModelToKey = {};
        this.persistReducers = {};
        this.serializedStrings = {};
        this.subscription = [];
        this.readyEvents = [];
        this.restoreTimer = -1;
        this.storeHelper = storeHelper;
        this.schema = {
            __persist: { version: '' },
        };
    }
    Persist.prototype.setConfig = function (config) {
        var _this = this;
        this.config = config;
        if (!config) {
            return this.onBootstrapped();
        }
        switch (config.storage) {
            case 'memory':
                this.storage = memoryStorage_1.default;
                break;
            default:
                this.storage = config.storage;
        }
        var allowlist = config.allowlist, version = config.version;
        this.bootstrapped = false;
        this.schema.__persist.version = version;
        this.allowKeys = [];
        this.mapFromModelToKey = {};
        Object.keys(allowlist).forEach(function (key) {
            var model = allowlist[key];
            _this.allowKeys.push(key);
            _this.mapFromModelToKey[typeof model === 'string' ? model : model.getReducerName()] = key;
        });
        return this;
    };
    Persist.prototype.rehydrate = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var data, tempReducers_1, shouldRestore_1;
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.bootstrapped || !this.config) {
                            return [2 /*return*/];
                        }
                        return [4 /*yield*/, this.storage.getItem(this.keyPrefix + this.config.key)];
                    case 1:
                        data = _a.sent();
                        if (data === null) {
                            return [2 /*return*/, this.resetAndRestore().onParsed().onBootstrapped()];
                        }
                        try {
                            tempReducers_1 = JSON.parse(data);
                            if (tempReducers_1.__persist.version === this.config.version) {
                                shouldRestore_1 = false;
                                delete tempReducers_1.__persist;
                                this.persistReducers = {};
                                Object.keys(tempReducers_1).forEach(function (key) {
                                    if (_this.allowKeys.indexOf(key) >= 0) {
                                        _this.serializedStrings[key] = tempReducers_1[key];
                                        _this.persistReducers[key] = JSON.parse(tempReducers_1[key]);
                                    }
                                    else {
                                        shouldRestore_1 = shouldRestore_1 || true;
                                    }
                                });
                                if (shouldRestore_1) {
                                    this.restore();
                                }
                            }
                            else {
                                this.resetAndRestore();
                            }
                        }
                        catch (e) {
                            this.resetAndRestore();
                            console.error('Unable to parser persist reducers from storage: ' + e.message);
                        }
                        this.onParsed().onBootstrapped();
                        return [2 /*return*/];
                }
            });
        });
    };
    Persist.prototype.update = function (nextState, force) {
        var _this = this;
        if (!this.config) {
            return;
        }
        // Persist is not ready before dispatch action TYPE_REHYDRATE
        if (!this.bootstrapped && !force) {
            return;
        }
        var tempState = tslib_1.__assign({}, this.persistReducers);
        var changed = false;
        Object.keys(this.mapFromModelToKey).forEach(function (reducerName) {
            var key = _this.mapFromModelToKey[reducerName];
            tempState[key] = nextState[reducerName];
            if (nextState[reducerName] !== _this.persistReducers[key]) {
                var tempString = JSON.stringify(nextState[reducerName]);
                changed = changed || _this.serializedStrings[key] !== tempString;
                _this.serializedStrings[key] = tempString;
            }
        });
        this.persistReducers = tempState;
        if (changed) {
            this.restore();
        }
    };
    Persist.prototype.isReady = function () {
        return this.bootstrapped;
    };
    Persist.prototype.getPersistData = function (reducerName, state) {
        if (!this.config) {
            return state;
        }
        var key = this.mapFromModelToKey[reducerName];
        if (!key) {
            return state;
        }
        if (!this.bootstrapped) {
            if (this.subscription.indexOf(reducerName) === -1) {
                this.subscription.push(reducerName);
            }
            return state;
        }
        var persistState = this.persistReducers[key];
        return persistState === undefined ? state : persistState;
    };
    Persist.prototype.onParsed = function () {
        var _this = this;
        if (this.subscription.length) {
            var payload_1 = {};
            var canDispatch_1 = false;
            this.subscription.forEach(function (reducerName) {
                var persistKey = _this.mapFromModelToKey[reducerName];
                if (persistKey) {
                    canDispatch_1 = true;
                    payload_1[reducerName] = _this.persistReducers[persistKey];
                }
            });
            this.subscription = [];
            if (canDispatch_1) {
                this.storeHelper.dispatch({
                    type: exports.TYPE_REHYDRATE,
                    payload: payload_1,
                });
            }
        }
        return this;
    };
    Persist.prototype.resetAndRestore = function () {
        this.persistReducers = {};
        this.subscription = [];
        this.restore();
        return this;
    };
    Persist.prototype.onBootstrapped = function () {
        this.bootstrapped = true;
        if (this.readyEvents.length) {
            this.readyEvents.forEach(function (item) { return item(); });
            this.readyEvents = [];
        }
        return this;
    };
    Persist.prototype.listen = function (fn) {
        var _this = this;
        this.readyEvents.push(fn);
        if (this.bootstrapped) {
            setTimeout(function () {
                if (_this.bootstrapped) {
                    _this.onBootstrapped();
                }
            });
        }
        return function () {
            _this.readyEvents = _this.readyEvents.filter(function (item) { return item !== fn; });
        };
    };
    ;
    Persist.prototype.restore = function () {
        var _this = this;
        if (!this.config) {
            return this;
        }
        var key = this.config.key;
        var strings = {};
        // Restore existing reducers
        Object.keys(this.persistReducers).forEach(function (key) {
            strings[key] = _this.serializedStrings[key];
        });
        var timer = this.restoreTimer = setTimeout(function () {
            if (timer !== _this.restoreTimer) {
                return;
            }
            _this.storage.setItem(_this.keyPrefix + key, JSON.stringify(tslib_1.__assign(tslib_1.__assign({}, strings), _this.schema)));
        });
        return this;
    };
    ;
    return Persist;
}());
exports.Persist = Persist;
